S3method("[",bg_GLM)
S3method("[",brainGraphList)
S3method("[",brainGraph_resids)
S3method("[",corr_mats)
S3method(anova,bg_GLM)
S3method(case.names,NBS)
S3method(case.names,bg_GLM)
S3method(case.names,brainGraph_resids)
S3method(case.names,mtpc)
S3method(coef,bg_GLM)
S3method(coeff_var,array)
S3method(coeff_var,default)
S3method(coeff_var,matrix)
S3method(confint,bg_GLM)
S3method(cooks.distance,bg_GLM)
S3method(deviance,bg_GLM)
S3method(df.residual,NBS)
S3method(df.residual,bg_GLM)
S3method(df.residual,mtpc)
S3method(dfbeta,bg_GLM)
S3method(dfbetas,bg_GLM)
S3method(extractAIC,bg_GLM)
S3method(fitted,bg_GLM)
S3method(formula,NBS)
S3method(formula,bg_GLM)
S3method(formula,mtpc)
S3method(groups,brainGraphList)
S3method(groups,brainGraph_resids)
S3method(groups,corr_mats)
S3method(hatvalues,bg_GLM)
S3method(influence,bg_GLM)
S3method(inv,array)
S3method(inv,list)
S3method(inv,matrix)
S3method(inv,qr)
S3method(labels,NBS)
S3method(labels,bg_GLM)
S3method(labels,mtpc)
S3method(logLik,bg_GLM)
S3method(make_brainGraph,bg_mediate)
S3method(make_brainGraph,igraph)
S3method(make_brainGraph,matrix)
S3method(make_brainGraphList,NBS)
S3method(make_brainGraphList,array)
S3method(make_brainGraphList,bg_GLM)
S3method(make_brainGraphList,corr_mats)
S3method(make_brainGraphList,mtpc)
S3method(nobs,NBS)
S3method(nobs,bg_GLM)
S3method(nobs,brainGraphList)
S3method(nobs,brainGraph_resids)
S3method(nobs,mtpc)
S3method(nregions,NBS)
S3method(nregions,bg_GLM)
S3method(nregions,brainGraph_resids)
S3method(nregions,corr_mats)
S3method(nregions,mtpc)
S3method(plot,IC)
S3method(plot,bg_GLM)
S3method(plot,brainGraph)
S3method(plot,brainGraphList)
S3method(plot,brainGraph_GLM)
S3method(plot,brainGraph_NBS)
S3method(plot,brainGraph_boot)
S3method(plot,brainGraph_mediate)
S3method(plot,brainGraph_mtpc)
S3method(plot,brainGraph_permute)
S3method(plot,brainGraph_resids)
S3method(plot,corr_mats)
S3method(plot,mtpc)
S3method(print,bg_GLM)
S3method(print,brainGraphList)
S3method(print,infl.bg_GLM)
S3method(print,summary.IC)
S3method(print,summary.NBS)
S3method(print,summary.bg_GLM)
S3method(print,summary.bg_mediate)
S3method(print,summary.brainGraph)
S3method(print,summary.brainGraph_boot)
S3method(print,summary.brainGraph_permute)
S3method(print,summary.brainGraph_resids)
S3method(print,summary.mtpc)
S3method(qr,array)
S3method(region.names,bg_GLM)
S3method(region.names,brainGraph_resids)
S3method(region.names,corr_mats)
S3method(region.names,data.table)
S3method(region.names,mtpc)
S3method(residuals,bg_GLM)
S3method(rstandard,bg_GLM)
S3method(rstudent,bg_GLM)
S3method(sigma,bg_GLM)
S3method(summary,IC)
S3method(summary,NBS)
S3method(summary,bg_GLM)
S3method(summary,bg_mediate)
S3method(summary,brainGraph)
S3method(summary,brainGraph_boot)
S3method(summary,brainGraph_permute)
S3method(summary,brainGraph_resids)
S3method(summary,mtpc)
S3method(symmetrize,array)
S3method(symmetrize,matrix)
S3method(terms,NBS)
S3method(terms,bg_GLM)
S3method(terms,mtpc)
S3method(variable.names,NBS)
S3method(variable.names,bg_GLM)
S3method(variable.names,mtpc)
S3method(vcov,bg_GLM)
export(NBS)
export(analysis_random_graphs)
export(aop)
export(apply_thresholds)
export(as_atlas)
export(as_brainGraphList)
export(bg_to_mediate)
export(brainGraph_GLM)
export(brainGraph_GLM_design)
export(brainGraph_boot)
export(brainGraph_mediate)
export(brainGraph_permute)
export(centr_betw_comm)
export(centr_lev)
export(check_sID)
export(coeff_determ)
export(coeff_table)
export(coeff_var)
export(colMax)
export(colMaxAbs)
export(colMin)
export(communicability)
export(contract_brainGraph)
export(cor.diff.test)
export(corr.matrix)
export(count_homologous)
export(count_inter)
export(covratio.bg_GLM)
export(create_atlas)
export(create_mats)
export(dffits_bg_GLM)
export(diag_sq)
export(edge_asymmetry)
export(edge_spatial_dist)
export(efficiency)
export(fastLmBG)
export(fastLmBG_3d)
export(fastLmBG_3dY)
export(fastLmBG_3dY_1p)
export(fastLmBG_f)
export(fastLmBG_t)
export(gateway_coeff)
export(get.resid)
export(get_thresholds)
export(graph_attr_dt)
export(guess_atlas)
export(hubness)
export(import_scn)
export(inv)
export(is.brainGraph)
export(is.brainGraphList)
export(is_binary)
export(loo)
export(make_auc_brainGraph)
export(make_brainGraph)
export(make_brainGraphList)
export(make_ego_brainGraph)
export(make_empty_brainGraph)
export(make_intersection_brainGraph)
export(mean_distance_wt)
export(mtpc)
export(nregions)
export(pad_zeros)
export(part_coeff)
export(partition)
export(pinv)
export(plot_brainGraph_multi)
export(plot_global)
export(plot_rich_norm)
export(plot_vertex_measures)
export(plot_volumetric)
export(qr_Q2)
export(qr_R2)
export(randomise)
export(randomise_3d)
export(region.names)
export(rich_club_all)
export(rich_club_attrs)
export(rich_club_coeff)
export(rich_club_norm)
export(rich_core)
export(robustness)
export(s_core)
export(set_brainGraph_attr)
export(sim.rand.graph.clust)
export(sim.rand.graph.hqs)
export(sim.rand.graph.par)
export(slicer)
export(small.world)
export(symm_mean)
export(symmetrize)
export(vertex_attr_dt)
export(vertex_spatial_dist)
export(vif.bg_GLM)
export(vulnerability)
export(within_module_deg_z_score)
export(write_brainnet)
export(xfm.weights)

importFrom(MASS,Null,fractions,ginv)
importFrom(Matrix,colSums,rowSums)
importFrom(abind,abind,adrop)
importFrom(doParallel,registerDoParallel)
importFrom(grid,gpar,grid.draw,grid.newpage,textGrob)
importFrom(methods,hasArg)
importFrom(parallel, clusterEvalQ, makeCluster)
importFrom(permute,shuffleSet)
import(igraph)
import(data.table)
import(foreach)
import(lattice)
importFrom(graphics, abline, image, layout, legend, lines, mtext, panel.smooth, par, plot,
  points, polygon, strheight, strwidth, text, title)
importFrom(grDevices, adjustcolor, colorRampPalette, dev.copy, dev.cur,
  dev.off, dev.set, extendrange, nclass.FD, png, pdf, gray)
importFrom(stats, anova, as.dist, as.formula, case.names, coef, complete.cases, confint,
  cooks.distance, cov.wt, cov2cor, covratio, density, deviance, df.residual, dfbeta, dfbetas, dffits,
  extractAIC, fitted, formula, hatvalues, influence, lm, logLik, median, nobs, p.adjust, pf,
  pnorm, ppoints, printCoefmat, pt, qf, qnorm, qqline, qqnorm, qt, quantile, residuals, rnorm, rstandard,
  rstudent, runif, sd, setNames, sigma, t.test, terms, var, variable.names, vcov)
importFrom(utils, combn, data, file_test, flush.console, getTxtProgressBar, hasName, head, packageVersion,
  setTxtProgressBar, tail, txtProgressBar)
