% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/random_graphs.R
\name{choose.edges}
\alias{choose.edges}
\title{Select edges for re-wiring}
\usage{
choose.edges(A, degs.large)
}
\arguments{
\item{A}{Numeric (adjacency) matrix}

\item{degs.large}{Integer vector of vertex numbers with degree greater than
one}
}
\value{
A data frame with four elements; two edges \code{(y1, z1)} and
  \code{(y2, z2)} will be removed, and two edges \code{(y1, y2)} and
  \code{(z1, z2)} will be added between the four vertices.
}
\description{
\code{choose.edges} selects edges to be re-wired when simulating random
graphs while controlling for \emph{clustering}. It is based on the algorithm
by Bansal et al. (2009), BMC Bioinformatics.
}
\author{
Christopher G. Watson, \email{cgwatson@bu.edu}
}
\keyword{internal}
