% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{package}
\name{brainGraph}
\alias{brainGraph-package}
\alias{brainGraph}
\alias{brainGraph-options}
\title{Default options for brainGraph}
\description{
brainGraph is a package for performing \emph{graph theory analysis} of brain
MRI data.
}
\section{Package options}{


brainGraph uses the following \code{\link{options}} to configure behavior:
\itemize{
  \item \code{bg.subject_id}: character string specifying the name your
    project/study uses as a subject identifier. All imported data (e.g.,
    covariates tables) \emph{MUST} have a column matching this. One possible
    alternative is \code{'participant_id'}, recommended by BIDS. Default:
    \code{'Study.ID'}
  \item \code{bg.group}: character string specifying the name your
    project/study uses as a group identifier. All imported data (e.g.,
    covariates tables) \emph{MUST} have a column matching this. One possible
    alternative is \code{'group'}, recommended by BIDS. Default:
    \code{'Group'}
  \item \code{bg.session}: character string specifying the name your
    project/study uses as a \dQuote{time} or session identifier, in the case
    of longitudinal studies. All imported data (e.g., covariates tables)
    \emph{MUST} have a column matching this. One possible alternative is
    \code{'session_id'}, recommended by BIDS. Default: \code{'Time'}
  \item \code{bg.progress}: logical indicating whether to show progress bars
    for functions that provide the option. Default: \code{TRUE}
  \item \code{bg.ncpus}: integer indicating the number of cores to use for
    parallel operations. Only used if you have not already registered a
    parallel backend (see Chapter 5 of the User Guide or
    \url{https://github.com/cwatson/brainGraph/blob/master/README.md} for
    examples). Default: \code{2L}
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/cwatson/brainGraph}
  \item Report bugs at \url{https://groups.google.com/forum/?hl=en#!forum/brainGraph-help}
}

}
\author{
\strong{Maintainer}: Christopher G. Watson \email{cgwatson@bu.edu} (\href{https://orcid.org/0000-0002-7082-7631}{ORCID})

}
